<?php

declare(strict_types=1);

namespace Drupal\Tests\lms_messages\Kernel;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\grequest\Plugin\Group\Relation\GroupMembershipRequest;
use Drupal\lms\Entity\CourseStatusInterface;
use Drupal\message\MessageInterface;

/**
 * Tests creation of messages.
 *
 * @group lms_messages
 */
final class LmsMessagesTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'field',
    'filter',
    'flexible_permissions',
    'group',
    'grequest',
    'lms',
    'lms_membership_request',
    'lms_messages',
    'message',
    'message_notify',
    'options',
    'state_machine',
    'text',
    'user',
    'views',
  ];

  /**
   * Entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('action');
    $this->installConfig(['group']);
    $this->installConfig(['grequest']);
    $this->installConfig(['lms']);
    $this->installConfig(['lms_membership_request']);
    $this->installConfig(['lms_messages']);
    $this->installEntitySchema('user');
    $this->installEntitySchema('group');
    $this->installEntitySchema('group_relationship');
    $this->installEntitySchema('group_config_wrapper');
    $this->installEntitySchema('message');
    $this->installEntitySchema('lms_course_status');
    $this->installEntitySchema('lms_lesson_status');
    $this->installEntitySchema('lms_answer');

    $this->entityTypeManager = $this->container->get(EntityTypeManagerInterface::class);
  }

  /**
   * Tests creation of messages.
   */
  public function testLmsMessages(): void {
    $teacher = $this->createUser();
    /** @var \Drupal\lms\Entity\Bundle\Course */
    $course = $this->entityTypeManager->getStorage('group')->create([
      'type' => 'lms_course',
      'name' => 'Test course',
      'uid' => $teacher->id(),
    ]);
    $course->save();
    $course->addMember($teacher);
    $student = $this->createUser();

    // Course needs evaluation.
    /** @var \Drupal\lms\Entity\CourseStatusInterface */
    $course_status = $this->entityTypeManager->getStorage('lms_course_status')->create([
      'uid' => $student->id(),
      'gid' => $course->id(),
      'score' => 0,
      'status' => CourseStatusInterface::STATUS_PROGRESS,
    ]);
    $course_status->save();
    $course_status->setStatus(CourseStatusInterface::STATUS_NEEDS_EVALUATION);
    $course_status->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), 'course_needs_evaluation');
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('user')->target_id, $student->id());
    self::assertEquals($message->get('uid')->target_id, $teacher->id());

    // Course evaluated.
    $course_status->setStatus(CourseStatusInterface::STATUS_PASSED);
    $course_status->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), 'course_evaluated');
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('uid')->target_id, $student->id());

    // New membership request.
    $course->removeMember($student);
    /** @var \Drupal\group\Entity\GroupRelationshipInterface */
    $membership_request = $this->entityTypeManager->getStorage('group_relationship')->create([
      'gid' => $course->id(),
      'entity_id' => $student->id(),
      'label' => 'membership request',
      'uid' => $student->id(),
      // From lms_membership_request config.
      'type' => 'lms_course-996ae0cbc1a35b89222ab',
      'plugin_id' => 'group_membership_request',
      GroupMembershipRequest::STATUS_FIELD => GroupMembershipRequest::REQUEST_NEW,
    ]);
    $membership_request->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), 'new_membership_request');
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('user')->target_id, $student->id());
    self::assertEquals($message->get('uid')->target_id, $teacher->id());

    // Membership request approved.
    $membership_request->set(GroupMembershipRequest::STATUS_FIELD, GroupMembershipRequest::REQUEST_APPROVED)->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), 'membership_request_approved');
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('uid')->target_id, $student->id());
  }

  /**
   * Get last message created.
   */
  private function getLastMessage(): MessageInterface {
    $message_storage = $this->entityTypeManager->getStorage('message');
    $results = $message_storage->getQuery()
      ->accessCheck(FALSE)
      ->sort('mid', 'DESC')
      ->range(0, 1)
      ->execute();
    self::assertEquals(1, \count($results));
    $message = $message_storage->load(\reset($results));
    \assert($message instanceof MessageInterface);
    return $message;
  }

}
